@@ -1,14 +1,11 @@
-#ifndef lint
-static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/events.c,v 1.3 1998/02/12 03:08:37 unammx Exp $";
-#endif
 /*****
 * events.c : HTML4.0 event routines
 *
-* This file Version	$Revision: 1.3 $
+* This file Version	$Revision: 1.4 $
 *
 * Creation date:		Fri Nov 14 14:48:28 GMT+0100 1997
-* Last modification: 	$Date: 1998/02/12 03:08:37 $
-* By:					$Author: unammx $
+* Last modification: 	$Date: 1999/07/29 01:26:28 $
+* By:					$Author: sopwith $
 * Current State:		$State: Exp $
 *
 * Author:				newt
@@ -36,6 +33,11 @@ static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/events
 /*****
 * ChangeLog 
 * $Log: events.c,v $
+* Revision 1.4  1999/07/29 01:26:28  sopwith
+*
+*
+* Fix all warnings.
+*
 * Revision 1.3  1998/02/12 03:08:37  unammx
 * Merge to Koen's XmHTML 1.1.2 + following fixes:
 *
