@@ -1,3 +1,4 @@
+#ifndef GNOME_EXCLUDE_DEPRECATED
 #ifndef __GTK_XMHTML_H__
 #define __GTK_XMHTML_H__
 
@@ -8,9 +9,9 @@
 #ifndef WITH_GTK
 # define WITH_GTK
 #endif
+
 #include <gtk-xmhtml/toolkit.h>
 #include <gtk-xmhtml/XmHTML.h>
-#include <gtk-xmhtml/XmHTMLP.h>
 #endif
 
 #ifdef __cplusplus
@@ -28,6 +29,10 @@ extern "C" {
 typedef struct _GtkXmHTML       GtkXmHTML;
 typedef struct _GtkXmHTMLClass  GtkXmHTMLClass;
 
+#ifndef GTK_XMHTML_LIBRARY
+#include <gtk-xmhtml/XmHTMLP.h>
+#endif
+
 struct _GtkXmHTML
 {
 	GtkContainer widget;
@@ -155,7 +160,7 @@ void gtk_xmhtml_set_mime_type                     (GtkXmHTML *html, char *mime_t
 void gtk_xmhtml_set_alpha_processing              (GtkXmHTML *html, int flag);
 void gtk_xmhtml_set_rgb_conv_mode                 (GtkXmHTML *html, int val);
 
-void Toolkit_Draw_Shadows(XmHTMLWidget w, GC* top_shadow, GC* bottom_shadow,
+void Toolkit_Draw_Shadows(XmHTMLWidget w, TGC top_shadow, TGC bottom_shadow,
 			  gint xs, gint ys, gint xe, gint ye, gint xxx, gint shadow_type);
 
 
@@ -179,3 +184,4 @@ typedef struct {
 } gtk_xmhtml_callback_info;
 
 #endif
+#endif /* GNOME_EXCLUDE_DEPRECATED */
