@@ -68,6 +68,24 @@ title_changed_event (ZvtTerm *term, VTTITLE_TYPE type, char *newtitle)
   }
 }
 
+static void
+button_press_event (ZvtTerm *term, GdkEventButton *e)
+{
+  int x,y;
+  GdkModifierType mask;
+  char *data, *str;
+
+  gdk_window_get_pointer(GTK_WIDGET(term)->window, &x, &y, &mask);
+
+  str = zvt_term_match_check(term, x/term->charwidth, y/term->charheight, &data);
+  if (str) {
+    gtk_signal_emit_stop_by_name (GTK_OBJECT (term), "button_press_event");
+    printf("User clicked on type '%s' -> '%s'\n", data, str);
+    return TRUE;
+  }
+  return FALSE;
+}
+
 static void
 set_hints (GtkWidget *widget)
 {
@@ -192,6 +210,16 @@ main (gint argc, gchar *argv[])
   zvt_term_set_scroll_on_output (ZVT_TERM (term), FALSE);
   zvt_term_set_background (ZVT_TERM (term), NULL, 0, 0);
   zvt_term_set_wordclass (ZVT_TERM (term), "-A-Za-z0-9/_:.,?+%=");
+
+  zvt_term_match_add (ZVT_TERM (term), "http://[^ ]*[^\\.]", VTATTR_UNDERLINE, "http");
+  zvt_term_match_add (ZVT_TERM (term), "ftp://[^ ]*[^\\.]", VTATTR_BOLD, "ftp");
+  zvt_term_match_add (ZVT_TERM (term), "file://[^ ]*[^\\.]", VTATTR_REVERSE, "file");
+
+  gtk_signal_connect (
+      GTK_OBJECT (term),
+      "button_press_event",
+      GTK_SIGNAL_FUNC (button_press_event),
+      NULL);
   
   gtk_signal_connect (
       GTK_OBJECT (term),
